<?php

/**
 * Description of AutoOpenGraph
 *
 * @version  1.0
 * @version $Rev: 13 $
 * @version $Date: 2012-04-19 21:32:07 +0200 (to, 19 apr 2012) $
 * @version $Id: autoopengraph.php 13 2012-04-19 19:32:07Z webbochsant@gmail.com $
 * @author Daniel Eliasson (joomla at stilero.com)
 * @copyright  (C) 2012-apr-15 Stilero Webdesign http://www.stilero.com
 * @category Plugins
 * @license	GPLv3
 * 
 * Joomla! is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * 
 * This file is part of autoopengraph.
 * 
 * AutoOpenGraph is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * AutoOpenGraph is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with AutoOpenGraph.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

// import library dependencies
jimport('joomla.plugin.plugin');

class plgContentAutoopengraph extends JPlugin {
    
    var $Article;
    var $ogTagsAdded;

    function plgContentAutoopengraph(&$subject, $config) {
        parent::__construct($subject, $config);
        $language = JFactory::getLanguage();
        $language->load('plg_system_autoopengraph', JPATH_ADMINISTRATOR, 'en-GB', true);
        $language->load('plg_system_autoopengraph', JPATH_ADMINISTRATOR, null, true);
        $this->errorOccured = FALSE;
    }
    
    /**
     *
     * @param type $article
     * @param type $params
     * @param type $limitstart
     * @return string 
     * @since 1.5
     */
    function onBeforeDisplayContent( &$article, &$params, $limitstart=0 ){
        //var_dump($article);exit;
        $this->setArticleFromJ15($article);
        $this->insertOGTags($article);
        return '';
    }
    
    /**
     * 
     * @param type $context
     * @param type $article
     * @param type $params
     * @param type $limitstart
     * @return string 
     * @since 1.6
     */
    function onContentBeforeDisplay ($context, &$article, &$params, $limitstart=0){
        $this->setArticleFromJ16($article);
        $this->insertOGTags($article);
        return '';
    }
    
    private function insertOGTags($articleObj){
        if( $this->ogTagsAdded ){
            return;
        }
        $document = JFactory::getDocument();
        $this->addOGTitle($document);
        $this->addOGType($document);
        $this->addOGDesc($document);
        $this->addOGLocale($document);
        $this->addOGSiteName($document);
        $this->addOGImage($document);
        $this->addOGUrl($document);
        $this->addOGExtra($document);
        $this->addOGFBAppID($document);
        $this->ogTagsAdded = true;
    }
    private function addOGExtra($document){
        $metaProps = array(
            '<meta property="article:published_time" content="'.$this->Article->publish_up.'" />',
            '<meta property="article:modified_time" content="'.$this->Article->modified.'" />',
            '<meta property="article:section" content="'.htmlentities(strip_tags($this->Article->category)).'" />'
            );
            foreach ($metaProps as $metaProp) {
                $document->addCustomTag($metaProp);
            }
    }
    
    private function addOGFBAppID($document){
        if( $this->params->def('og-fbappid')!="" ){
            $metaProp = '<meta property="fb:app_id" content="'.$this->params->def('og-fbappid').'" />';
            $document->addCustomTag($metaProp);
        }
    }   
    
    private function addOGTitle($document){
        $metaProp = '';
        switch ($this->params->def('og-title')) {
            case 1:
                $metaProp = '<meta property="og:title" content="'.$this->Article->title.'" />';
                break;
            case 2:
                $metaProp = '<meta property="og:title" content="'.$this->params->def('og-title-custom').'" />';
                break;
            default:
                return;
                break;
        }
        $document->addCustomTag($metaProp);
    }
    
    private function addOGType($document){
        $metaProp = '';
        switch ($this->params->def('og-type')) {
            case 1:
                $metaProp = '<meta property="og:type" content="article" />';
                break;
            case 2:
                $metaProp = '<meta property="og:type" content="website" />';
                break;
            case 3:
                $metaProp = '<meta property="og:type" content="book" />';
                break;
            case 4:
                $metaProp = '<meta property="og:type" content="profile" />';
                break;
            case 5:
                $metaProp = '<meta property="og:type" content="'.$this->params->def('og-type-custom').'" />';
                break;
            default:
                return;
                break;
        }
        $document->addCustomTag($metaProp);
    }
    
    private function addOGDesc($document){
        $metaProp = '';
        switch ($this->params->def('og-desc')) {
            case 1:
                $metaProp = '<meta property="og:description" content="'.htmlentities(strip_tags($this->Article->metadesc)).'" />';
                break;
            case 2:
                $metaProp = '<meta property="og:description" content="'.htmlentities(strip_tags($this->Article->introtext)).'" />';
                break;
            case 3:
                $joomlaConfig = JFactory::getConfig();
                $joomlaSiteName = $joomlaConfig->getValue( 'config.MetaDesc' );
                $metaProp = '<meta property="og:description" content="'.htmlentities(strip_tags($joomlaSiteName)).'" />';
                break;
            case 4:
                $metaProp = '<meta property="og:description" content="'.htmlentities(strip_tags($this->params->def('og-desc-custom'))).'" />';
                break;
            default:
                return;
                break;
        }
        $document->addCustomTag($metaProp);
    }
    
    private function addOGLocale($document){
        $langObj = JFactory::getLanguage();
        $currentLang = str_replace( "-", "_", $langObj->getTag() );
        switch ($this->params->def('og-locale')) {
            case 1:
                $ogContent = $currentLang;
                break;
            case 2:
                $ogContent = $currentLang;
                break;
            case 3:
                $ogContent = str_replace( "-", "_", $this->params->def('og-locale-custom') );
                break;
            default:
                return;
                break;
        }
        $metaProp = '<meta property="og:locale" content="'.htmlentities(strip_tags($ogContent)).'" />';
        $document->addCustomTag($metaProp);
    }
    
    private function addOGSiteName($document){
        $joomlaConfig = JFactory::getConfig();
        $joomlaSiteName = $joomlaConfig->getValue( 'config.sitename' );
        switch ($this->params->def('og-sitename')) {
            case 1:
                $ogContent = $joomlaSiteName;
                break;
            case 2:
                $ogContent = $this->params->def('og-sitename-custom');
                break;
            default:
                return;
                break;
        }
        $metaProp = '<meta property="og:site_name" content="'.htmlentities(strip_tags($ogContent)).'" />';
        $document->addCustomTag($metaProp);
    }
    
    private function getOGImage($option){
        $images = $this->Article->contentImages;
        switch ($option) {
            case 1:
                $image = (isset($images[0]['src'])) ? $images[0]['src'] : '';
                break;
            case 2:
                $image = (isset($this->Article->imageIntro)) ? $this->Article->imageIntro : '';
                break;
            case 3:
                $image = (isset($this->Article->imageFull)) ? $this->Article->imageFull : '';
                break;
            case 4:
                $cssClass = $this->params->def('og-img-class');
                $classImage = $this->getImageWithClass($images, $cssClass);
                $image = (isset($classImage)) ? $classImage : '';
                break;
            case 5:
                if($this->params->def('og-img-custom') != ''){
                    $image = 'images/'.$this->params->def('og-img-custom');
                }
                break;
            default:
                return;
                break;
        }
        if($image != ""){
            $image = preg_match('/http/', $image)? $image : JURI::root().$image;
            return $image;
        }
    }

    private function addOGImage($document){
        $image = $this->getOGImage($this->params->def('og-img-prio1'));
        if($image == "" ){
            $image = $this->getOGImage($this->params->def('og-img-prio2'));
        }
        if($image == "" ){
            $image = $this->getOGImage($this->params->def('og-img-prio3'));
        }
        if($image != ""){
            $metaProp = '<meta property="og:image" content="'.htmlentities(strip_tags($image)).'" />';
            $document->addCustomTag($metaProp);
        }
    }
    
    private function addOGUrl($document){
        $url = $this->Article->fullURL;
        $parsedURL = parse_url(JURI::root());
        $url = preg_match('/http/', $url)? $url :  $parsedURL['scheme'].'://'.$parsedURL['host']. $url;
        $metaProp = '<meta property="og:url" content="'.$url.'" />';
        $document->addCustomTag($metaProp);
    }
    
    private function getMetaDataArray($articleObj){
        $imageSrc = $this->getImageSrcFromContent($articleObj); 
        $imageSrc = ( isset($imageSrc) )? $imageSrc : "";
        $descNeedles = array("\n", "\r", "\"", "'");
        $desc = (isset($articleObj->introtext) )? strip_tags( str_replace($descNeedles, "", $articleObj->introtext )) : "";
        $joomlaConfig = JFactory::getConfig();
        $joomlaSiteName = $joomlaConfig->getValue( 'config.sitename' );
        $langObj = JFactory::getLanguage();
        $currentLang = str_replace( "-", "_", $langObj->getTag() );
        $metaData = array(
            'og:title'          =>  $articleObj->title,
            'og:image'          =>  $imageSrc,
            'og:description'    =>  $desc,
            'og:locale'         =>  $currentLang,
            'og:type'           =>  'website',
            'og:site_name'      =>  $joomlaSiteName,
            'og:url'            =>  $this->getFullURL($articleObj->id)
        );
        return $metaData;
    }

    
    private function setArticleFromJ15($j15article){
        $articleObj = new stdClass();
        $articleObj->title = $j15article->title;
        $articleObj->introtext = (isset($j15article->introtext))? $j15article->introtext : '';
        $articleObj->contentImages = (isset($j15article->text))? $this->getContentImages($j15article->text) : '';
        $articleObj->language= (isset($j15article->language))? $j15article->language : '';
        $articleObj->metadesc = (isset($j15article->metadesc))? $j15article->metadesc : '';
        $articleObj->fullURL = JRoute::_( 'index.php?view=article&catid=' . 
                $j15article->catslug . 
                '&id=' . $j15article->slug );
        $this->Article = $articleObj;
    }
    
    private function setArticleFromJ16($j16article){
        $articleObj = new stdClass();
        $articleObj->title = htmlentities($j16article->title);
        $articleObj->introtext = (isset($j16article->introtext))? $j16article->introtext : '';
        $images = (isset($j16article->images))? $this->getArticleImages($j16article->images) : '';
        $articleObj->imageIntro = (isset($images['intro']))? $images['intro'] : '';
        $articleObj->imageFull = (isset($images['full']))? $images['full'] : '';
        $articleObj->contentImages = (isset($j16article->text))? $this->getContentImages($j16article->text) : '';
        $articleObj->metadesc = (isset($j16article->metadesc))? $j16article->metadesc : '';
        $articleObj->fullURL = JRoute::_( 'index.php?view=article&catid=' . $j16article->catslug . '&id=' . $j16article->slug);
        $articleObj->category = $j16article->category_title;
        $articleObj->publish_up = $j16article->publish_up;
        $articleObj->publish_down = $j16article->publish_down;
        $articleObj->modified = $j16article->modified;
        $this->Article = $articleObj;
    }
    
    private function getArticleImages($imageJSON){
        $obj = json_decode($imageJSON);
        $introImage = ( isset( $obj->{'image_intro'} ) ) ? $obj->{'image_intro'} : '' ;
        $fullImage = ( isset ($obj->{'image_fulltext'}) )? $obj->{'image_fulltext'} : '';
        $images = array(
            'intro' => $introImage,
            'full'  => $fullImage
        );
        return $images;
    }
    
    private function getContentImages($contentSource){
        if( ($contentSource == '') || (!class_exists('DOMDocument')) ){
            return;
        }
        $html = new DOMDocument();
        $html->loadHTML($contentSource);
        $images = array();
        foreach($html->getElementsByTagName('img') as $image) {
            $images[] = array(
                'src' => $image->getAttribute('src'),
                'class' => $image->getAttribute('class'),
            );
        }
        return $images;
    }
    
    private function getImageWithClass($images, $cssClass){
        if( (!isset($images)) || (empty ($images)) ){
            return;
        }
        foreach ($images as $image) {
            if($image['class'] == $cssClass){
                return $image['src'];
            }
        }
    }
}
//End Class